@@ -2,10 +2,10 @@
 # There's no way to ignore "F401 '...' imported but unused" warnings in this
 # module, but to preserve other warnings. So, don't check this module at all.
 
-__version__ = "0.3.0"
+__version__ = "0.5.1"
 
 from .accelerator import Accelerator
 from .kwargs_handlers import DistributedDataParallelKwargs, GradScalerKwargs
 from .notebook_launcher import notebook_launcher
 from .state import DistributedType
-from .utils import synchronize_rng_states
+from .utils import DeepSpeedPlugin, synchronize_rng_states
